// Macro for event display; 
// Shyam Kumar; INFN Bari, Italy
// shyam.kumar@ba.infn.it, shyam.kumar@cern.ch


void Event_display_example()
{
	
   TEveManager::Create();
   
  TString rootfile="epic_brycecanyon_Shyam.root"; // put geometry name
  TEveManager::Create();
  gGeoManager = TGeoManager::Import(rootfile); // or use TGeoManager::Import(rootfile)
  if (gGeoManager == nullptr) return;
  TEveGeoTopNode *EPIC = new TEveGeoTopNode(gGeoManager,gGeoManager->GetTopNode()); // pass node here instead of topnode
  EPIC->SetVisLevel(5);
  
   // Set Transparency of Detectors
   TObjArray* allvolumes = gGeoManager->GetListOfVolumes();
    for(Int_t i=0; i<allvolumes->GetEntries();i++){
          TGeoVolume* vol= (TGeoVolume*)allvolumes->At(i);
          TString volname = vol->GetIconName();
          if (volname.Contains("SVTX")) vol->SetLineColor(kRed); 	
          else if (volname.Contains("BARR")) vol->SetLineColor(kBlue); 
          vol->SetTransparency(60); // set the transparency level for the volumes
    } 
  
  gEve->AddGlobalElement(EPIC);
  gEve->FullRedraw3D(kTRUE);
   
      // Text setting
   TEveText *text = new TEveText("ePIC: Shyam Kumar");
   text->PtrMainTrans()->RotateLF(1, 3, TMath::PiOver2());
   text->SetMainColor(kMagenta);
   text->SetFontSize(40);
   text->SetFontMode(TGLFont::kPixmap);
   text->SetLighting(kTRUE);
   text->RefMainTrans().SetPos(0.,400.,200.);
   gEve->AddElement(text);
   
   TEveTrackList *list = new TEveTrackList();
   list->SetElementName("MCTracks");
   
   Int_t propagator = 2; // 0 default, 1 RungeKutta, 2 kHelix
   TEveTrackPropagator* g_prop = 0;
   TEveTrackPropagator *prop = g_prop = list->GetPropagator(); //default propogator

   if (propagator==1)
   {
      prop->SetStepper(TEveTrackPropagator::kRungeKutta);
      list->SetName("RK Propagator");
   }
   else if (propagator==2)
   {
   	  prop->SetStepper(TEveTrackPropagator::kHelix);
      list->SetName("Heix Propagator");
   }
   prop->SetMagField(0.,0.,1.7); // 3.0 Tesla in Z-direction

  TEveRecTrackD *rc = new TEveRecTrackD(); // Reconstructed Track
  rc->fV.Set(0, 0, 0);
  rc->fP.Set(1.0, 1.0,0.5);
  rc->fSign = 1; // Charge
  TEveTrack *track = new TEveTrack(rc, prop);
  track->SetMarkerStyle(4);
  track->SetLineColor(kGreen);
  track->SetLineWidth(2);
  list->AddElement(track);
  track->MakeTrack();
  gEve->AddElement(list); 
  gEve->Redraw3D();

}


