// Macro for visualzing geometry using Eve
// Sham Kumar; shyam055119@gmail.com; shyam.kumar@ba.infn.it

void draw_geom()
{
	
  Bool_t draw_full_geo = true;
  Bool_t draw_transparent = true;
  // set first node = 0 and end node = -1 for full geometry if draw_full_geo set as false 
  int startnode = 3; int endnode = 9; 
  	
  TString rootfile="pfrich.gdml"; //pfrich.gdml, epic_brycecanyon_Shyam.root, epic_tracking_only.root put geometry name
  TEveManager::Create();
  gGeoManager = TGeoManager::Import(rootfile); // or use TGeoManager::Import(rootfile)
  if (gGeoManager == nullptr) return;
  
   // TopNode has many nodes access this way
   TObjArray* allnodes = gGeoManager->GetTopNode()->GetNodes(); 
   const int nNodes = allnodes->GetEntries();
   if (endnode == -1) endnode = nNodes; 
  
    // Set Transparency of Detectors
   if (draw_transparent){ 
   TObjArray* allvolumes = gGeoManager->GetListOfVolumes();
    for(Int_t i=0; i<allvolumes->GetEntries();i++){
          TGeoVolume* vol= (TGeoVolume*)allvolumes->At(i);
          TString volname = vol->GetIconName();
          if (volname.Contains("SVTX")) vol->SetLineColor(kRed); 	
          else if (volname.Contains("BARR")) vol->SetLineColor(kBlue); 
          vol->SetTransparency(60); // set the transparency level for the volumes
    } 
    }  
   if (draw_full_geo){
   TEveGeoTopNode *EPIC_Tracker = new TEveGeoTopNode(gGeoManager,gGeoManager->GetTopNode());
   EPIC_Tracker->SetVisLevel(5);
   gEve->AddGlobalElement(EPIC_Tracker);  
   }
   else{
    for(Int_t i=startnode; i<endnode;i++){
    TGeoNode* node= (TGeoNode*)allnodes->At(i);
     TString volname = node->GetVolume()->GetName();
     TEveGeoTopNode *evenode = new TEveGeoTopNode(gGeoManager,node);
     evenode->SetVisLevel(5);
     gEve->AddGlobalElement(evenode);  
    }     
    }    
    gEve->FullRedraw3D(kTRUE);
    
    // Clip geometry    // Ax+By+CZ+D = 0 first 4 entry of data are A,B,C,D rest for the clipbox options
   // double data[6] = {0.,0.,10.0,0.,0.,0.}; // Cut w.r.t. Z = 10 plane
    double data[6] = {0.,1.,0.0,0.,0.,0.}; // Default cut at y = 1 plane
    TGLViewer  *viewer = gEve->GetDefaultGLViewer();
    viewer->GetClipSet()->SetClipType(TGLClip::kClipPlane); // Other options are kClipNone = 0, kClipPlane, kClipBox  
   // viewer->GetClipSet()->SetClipState(TGLClip::kClipPlane,data);
    Double_t center[3] ={0.,0.,0.};
    Double_t *cent; cent = center;
    viewer->SetOrthoCamera(TGLViewer::kCameraOrthoZOX,2.0, 1.0, cent,1.0,0.); // kCameraPerspXOZ, kCameraPerspYOZ, kCameraPerspXOY, kCameraOrthoXOY,
    viewer->SetGuideState(TGLUtil::kAxesOrigin, kTRUE, kFALSE, 0);
    viewer->RefreshPadEditor(viewer);	
  //viewer->CurrentCamera().RotateRad(-0.5, 0.5);
    viewer->DoDraw();


}


